@@ -112,12 +112,12 @@ class Simulator(QObject):
             self.sender_ready = True
 
     def stop(self, msg=""):
-        self.log_message("Stop simulation: " + "{}".format(msg))
-        self.is_simulating = False
+        self.simulation_stopped.emit()
 
-        if msg == "Finished":
-            # Ensure devices can send their last data before killing them
-            time.sleep(0.5)
+        if self.is_simulating:
+            self.log_message("Stop simulation" + (" ({})".format(msg.strip()) if msg else ""))
+            self.is_simulating = False
+            self.do_restart = False
 
         # stop devices
         if self.sniffer:
@@ -126,8 +126,6 @@ class Simulator(QObject):
         if self.sender:
             self.sender.stop()
 
-        self.simulation_stopped.emit()
-
     def restart(self):
         self.transcript.start_new_round()
         self.reset()
@@ -201,7 +199,7 @@ class Simulator(QObject):
 
     def __fill_counter_values(self, command: str):
         result = []
-        regex = "(item[0-9]+\.counter_value)"
+        regex = r"(item[0-9]+\.counter_value)"
         for token in re.split(regex, command):
             if re.match(regex, token) is not None:
                 try:
@@ -325,7 +323,7 @@ class Simulator(QObject):
             self.last_sent_message = msg
         else:
             # we have to receive a message
-            self.log_message("<i>Waiting for message {}</i>".format(msg.index()))
+            self.log_message("Waiting for message {}...".format(msg.index()))
             sniffer = self.sniffer
             if sniffer is None:
                 self.log_message("Fatal: No sniffer configured")
@@ -336,6 +334,7 @@ class Simulator(QObject):
             max_retries = self.project_manager.simulator_retries
             while self.is_simulating and not self.simulation_is_finished() and retry < max_retries:
                 received_msg = self.receive_message(sniffer)
+                self.log_message("  Received {} data bits".format(len(received_msg)))
 
                 if not self.is_simulating:
                     return
@@ -355,6 +354,7 @@ class Simulator(QObject):
                 received_msg.decoder = new_message.decoder
                 received_msg.message_type = new_message.message_type
 
+                self.log_message("  Check whether received data matches")
                 check_result, error_msg = self.check_message(received_msg, new_message, retry=retry,
                                                              msg_index=msg.index())
 
@@ -381,8 +381,10 @@ class Simulator(QObject):
         if isinstance(message, list) and len(message) > 0:
             self.log_messages.append(timestamp + ": " + message[0])
             self.log_messages.extend(message[1:])
+            logger.debug("\n".join(message))
         else:
             self.log_messages.append(timestamp + ": " + message)
+            logger.debug(message)
 
     def check_message(self, received_msg, expected_msg, retry: int, msg_index: int) -> (bool, str):
         if len(received_msg.decoded_bits) == 0:
@@ -461,8 +463,7 @@ class Simulator(QObject):
         if len(sniffer.messages) > 0:
             return sniffer.messages.pop(0)
 
-        spy = QSignalSpy(sniffer.message_sniffed)
-        if spy.wait(self.project_manager.simulator_timeout_ms):
+        if QSignalSpy(sniffer.message_sniffed).wait(self.project_manager.simulator_timeout_ms):
             try:
                 return sniffer.messages.pop(0)
             except IndexError:
@@ -504,8 +505,8 @@ class Simulator(QObject):
                 cmd = self.__fill_counter_values(lbl.external_program)
                 result = util.run_command(cmd, transcript, use_stdin=True)
                 if len(result) != lbl.end - lbl.start:
-                    log_msg = "Result value of external program {} ({}) does not match label length {}"
-                    logger.error(log_msg.format(result, len(result), lbl.end - lbl.start))
+                    log_msg = "Result value of external program {}: {} ({}) does not match label length {}"
+                    logger.error(log_msg.format(cmd, result, len(result), lbl.end - lbl.start))
                     continue
 
                 try:
